@@ -19,11 +19,13 @@ pub mod helpers;
 pub mod indenter;
 pub mod input;
 pub mod list;
+pub mod pager;
 pub mod partition;
 pub mod platform;
 pub mod redact;
 pub mod reporter;
 pub mod reuse_build;
+pub mod run_mode;
 pub mod runner;
 // TODO: move this module to the cargo-nextest crate and make it a private module once we get rid of
 // the tests in nextest-runner/tests/integration which depend on this to provide correct host and
@@ -39,6 +41,7 @@ mod time;
 #[cfg(feature = "self-update")]
 pub mod update;
 pub mod usdt;
+pub mod user_config;
 pub mod write_str;
 
 pub use rustc_cli::RustcCli;
